# -*- coding: utf-8 -*-

""" Unit test for pyKwalify - Rule """

# python std lib
import unittest

# 3rd party imports
import pytest

# pyKwalify imports
import pykwalify
from pykwalify.errors import RuleError, SchemaConflict
from pykwalify.rule import Rule
from pykwalify.compat import unicode


class TestRule(unittest.TestCase):

    def setUp(self):
        pykwalify.partial_schemas = {}

    def test_schema(self):
        # Test that when using both schema; and include tag that it throw an error because schema; tags should be parsed via Core()
        with pytest.raises(RuleError) as r:
            Rule(schema={"schema;str": {"type": "map", "mapping": {"foo": {"type": "str"}}}, "type": "map", "mapping": {"foo": {"include": "str"}}})
        assert str(r.value) == "<RuleError: error code 4: Schema is only allowed on top level of schema file: Path: '/'>"
        assert r.value.error_key == 'schema.not.toplevel'

    def test_unkknown_key(self):
        # Test that providing an unknown key raises exception
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "foobar": True})
        assert str(r.value) == "<RuleError: error code 4: Unknown key: foobar found: Path: '/'>"
        assert r.value.error_key == 'key.unknown'

    def test_matching_rule(self):
        # Test that exception is raised when a invalid matching rule is used
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "map", "matching-rule": "foobar", "mapping": {"regex;.+": {"type": "seq", "sequence": [{"type": "str"}]}}})
        assert str(r.value) == "<RuleError: error code 4: Specified rule in key: foobar is not part of allowed rule set : ['any', 'all']: Path: '/'>"
        assert r.value.error_key == 'matching_rule.not_allowed'

    def test_allow_empty_map(self):
        r = Rule(schema={"type": "map", "allowempty": True, "mapping": {"name": {"type": "str"}}})
        assert r.allowempty_map is True

    def test_type_value(self):
        # TODO: This test is currently semi broken, partial schemas might be somewhat broken possibly
        # # Test that when only having a schema; rule it should throw error
        # with pytest.raises(RuleError) as r:
        #     Rule(schema={"schema;fooone": {"type": "map", "mapping": {"foo": {"type": "str"}}}})
        # assert str(r.value) == "<RuleError: error code 4: Key 'type' not found in schema rule: Path: '/'>"
        # assert r.value.error_key == 'type.missing'

        # Test a valid rule with both "str" and "unicode" types work
        r = Rule(schema={"type": str("str")})
        r = Rule(schema={"type": unicode("str")})

        # Test that type key must be string otherwise exception is raised
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": 1})
        assert str(r.value) == "<RuleError: error code 4: Key 'type' in schema rule is not a string type (found int): Path: '/'>"
        assert r.value.error_key == 'type.not_string'

        # this tests that the type key must be a string
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": 1}, parent=None)
        assert str(r.value) == "<RuleError: error code 4: Key 'type' in schema rule is not a string type (found int): Path: '/'>"
        assert r.value.error_key == 'type.not_string'

    def test_name_value(self):
        with pytest.raises(RuleError) as r:
            Rule(schema={'type': 'str', 'name': {}})
        assert str(r.value) == "<RuleError: error code 4: Value: {} for keyword name must be a string: Path: '/'>"

    def test_nullable_value(self):
        # Test that nullable value must be bool otherwise exception is raised
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "nullable": "foobar"})
        assert str(r.value) == "<RuleError: error code 4: Value: 'foobar' for nullable keyword must be a boolean: Path: '/'>"
        assert r.value.error_key == 'nullable.not_bool'

    def test_desc_value(self):
        with pytest.raises(RuleError) as r:
            Rule(schema={'type': 'str', 'desc': []})
        assert str(r.value) == "<RuleError: error code 4: Value: [] for keyword desc must be a string: Path: '/'>"

    def test_example_value(self):
        with pytest.raises(RuleError) as r:
            Rule(schema={'type': 'str', 'example': []})
        assert str(r.value) == "<RuleError: error code 4: Value: [] for keyword example must be a string: Path: '/'>"

    def test_required_value(self):
        # Test that required value must be bool otherwise exception is raised
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "required": "foobar"})
        assert str(r.value) == "<RuleError: error code 4: Value: 'foobar' for required keyword must be a boolean: Path: '/'>"
        assert r.value.error_key == 'required.not_bool'

    def test_pattern_value(self):
        # this tests a invalid regexp pattern
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "pattern": "/@/\\"})
        assert str(r.value) == "<RuleError: error code 4: Syntax error when compiling regex pattern: None: Path: '/'>"
        assert r.value.error_key == 'pattern.syntax_error'

        # Test that pattern keyword is not allowed when using a map
        # with self.assertRaisesRegexp(RuleError, ".+map\.pattern.+"):
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "map", "pattern": "^[a-z]+$", "allowempty": True, "mapping": {"name": {"type": "str"}}})
        assert str(r.value) == "<RuleError: error code 4: Keyword pattern is not allowed inside map: Path: '/'>"
        assert r.value.error_key == 'pattern.not_allowed_in_map'

        # Test that pattern value must be string otherwise exception is raised
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "pattern": 1})
        assert str(r.value) == "<RuleError: error code 4: Value of pattern keyword: '1' is not a string: Path: '/'>"
        assert r.value.error_key == 'pattern.not_string'

    def test_date_and_format_value(self):
        r = Rule(schema={"type": "date", "format": "%y"})
        assert r.format is not None, "date var not set proper"
        assert isinstance(r.format, list), "date format should be a list"
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "date", "format": 1})
        assert str(r.value) == "<RuleError: error code 4: Value of format keyword: '1' must be a string or list or string values: Path: '/'>"
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "map", "format": "%y"})
        assert str(r.value) == "<RuleError: error code 4: Keyword format is only allowed when used with the following types: ('date',): Path: '/'>"

    def test_enum_value(self):
        # this tests the various valid enum types
        Rule(schema={"type": "int", "enum": [1, 2, 3]})
        Rule(schema={"type": "bool", "enum": [True, False]})
        r = Rule(schema={"type": "str", "enum": ["a", "b", "c"]})
        assert r.enum is not None, "enum var is not set proper"
        assert isinstance(r.enum, list), "enum is not set to a list"
        assert len(r.enum) == 3, "invalid length of enum entries"

        # this tests the missmatch between the type and the data inside a enum
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "enum": [1, 2, 3]})
        assert str(r.value).startswith("<RuleError: error code 4: Item: '1' in enum is not of correct class type:")
        assert r.value.error_key == 'enum.type.unmatch'

        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "enum": True})
        assert str(r.value) == "<RuleError: error code 4: Enum is not a sequence: Path: '/'>"
        assert r.value.error_key == 'enum.not_seq'

    def test_assert_value(self):
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "seq", "sequence": [{"type": "str", "assert": 1}]})
        assert str(r.value) == "<RuleError: error code 4: Value: '1' for keyword 'assert' is not a string: Path: '/sequence/0'>"
        assert r.value.error_key == 'assert.not_str'

        # Test that invalid characters is not present
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "seq", "sequence": [{"type": "str", "assert": "__import__"}]})
        assert str(r.value) == "<RuleError: error code 4: Value: '__import__' contain invalid content that is not allowed to be present in assertion keyword: Path: '/sequence/0'>"  # NOQA: E501
        assert r.value.error_key == 'assert.unsupported_content'

    def test_length(self):
        r = Rule(schema={"type": "int", "length": {"max": 10, "min": 1}})
        assert r.length is not None, "length var not set proper"
        assert isinstance(r.length, dict), "range var is not of dict type"

        # this tests that the range key must be a dict
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "int", "length": []})
        assert str(r.value) == "<RuleError: error code 4: Length value is not a dict type: '[]': Path: '/'>"
        assert r.value.error_key == 'length.not_map'

        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "length": {"max": "z"}})
        assert str(r.value) == "<RuleError: error code 4: Value: 'z' for 'max' keyword is not a number: Path: '/'>"
        assert r.value.error_key == 'length.max.not_number'

        # this tests that min is bigger then max that should not be possible
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "int", "length": {"max": 10, "min": 11}})
        assert str(r.value) == "<RuleError: error code 4: Value for 'max' can't be less then value for 'min'. 10 < 11: Path: '/'>"
        assert r.value.error_key == 'length.max_lt_min'

        # test that min-ex is bigger then max-ex, that should not be possible
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "int", "length": {"max-ex": 10, "min-ex": 11}})
        assert str(r.value) == "<RuleError: error code 4: Value for 'max-ex' can't be less then value for 'min-ex'. 10 <= 11: Path: '/'>"
        assert r.value.error_key == 'length.max-ex_le_min-ex'

        # test that a string has non negative boundaries
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "length": {"max": -1, "min": -2}})
        assert str(r.value) == "<RuleError: error code 4: Value for 'min' can't be negative in case of type str.: Path: '/'>"
        assert r.value.error_key == 'length.min_negative'

        # test that a seq has non negative boundaries
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "seq", "length": {"max": 3, "min": -2}})
        assert str(r.value) == "<RuleError: error code 4: Value for 'min' can't be negative in case of type seq.: Path: '/'>"
        assert r.value.error_key == 'length.min_negative'

    def test_range_value(self):
        r = Rule(schema={"type": "int", "range": {"max": 10, "min": 1}})
        assert r.range is not None, "range var not set proper"
        assert isinstance(r.range, dict), "range var is not of dict type"

        # this tests that the range key must be a dict
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "int", "range": []})
        assert str(r.value) == "<RuleError: error code 4: Range value is not a dict type: '[]': Path: '/'>"
        assert r.value.error_key == 'range.not_map'

        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "range": {"max": "z"}})
        assert str(r.value) == "<RuleError: error code 4: Value: 'z' for 'max' keyword is not a number: Path: '/'>"
        assert r.value.error_key == 'range.max.not_number'

        # this tests that min is bigger then max that should not be possible
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "int", "range": {"max": 10, "min": 11}})
        assert str(r.value) == "<RuleError: error code 4: Value for 'max' can't be less then value for 'min'. 10 < 11: Path: '/'>"
        assert r.value.error_key == 'range.max_lt_min'

        # test that min-ex is bigger then max-ex, that should not be possible
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "int", "range": {"max-ex": 10, "min-ex": 11}})
        assert str(r.value) == "<RuleError: error code 4: Value for 'max-ex' can't be less then value for 'min-ex'. 10 <= 11: Path: '/'>"
        assert r.value.error_key == 'range.max-ex_le_min-ex'

        # test that a string has non negative boundaries
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "range": {"max": -1, "min": -2}})
        assert str(r.value) == "<RuleError: error code 4: Value for 'min' can't be negative in case of type str.: Path: '/'>"
        assert r.value.error_key == 'range.min_negative'

        # test that a seq has non negative boundaries
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "seq", "range": {"max": 3, "min": -2}})
        assert str(r.value) == "<RuleError: error code 4: Value for 'min' can't be negative in case of type seq.: Path: '/'>"
        assert r.value.error_key == 'range.min_negative'

    def test_ident_value(self):
        pass

    def test_unique_value(self):
        # this tests that this cannot be used in the root level
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "str", "unique": True})
        assert str(r.value) == "<RuleError: error code 4: Keyword 'unique' can't be on root level of schema: Path: '/'>"
        assert r.value.error_key == 'unique.not_on_root_level'

        # this tests that unique cannot be used at root level
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "seq", "unique": True})
        assert str(r.value) == "<RuleError: error code 4: Type of the value: 'seq' for 'unique' keyword is not a scalar type: Path: '/'>"
        assert r.value.error_key == 'unique.not_scalar'

    def test_sequence(self):
        # this tests seq type with a internal type of str
        r = Rule(schema={"type": "seq", "sequence": [{"type": "str"}]})
        assert r.type is not None, "rule not contain type var"
        assert r.type == "seq", "type not 'seq'"
        assert r.sequence is not None, "rule not contain sequence var"
        assert isinstance(r.sequence, list), "rule is not a list"

        # Test basic sequence rule
        r = Rule(schema={"type": "seq", "sequence": [{"type": "str"}]})
        assert r.type == "seq"
        assert isinstance(r.sequence, list)
        assert isinstance(r.sequence[0], Rule)
        assert r.sequence[0].type == "str"

        # Test sequence without explicit type
        r = Rule(schema={"sequence": [{"type": "str"}]})
        assert r.type == "seq"
        assert isinstance(r.sequence, list)
        assert isinstance(r.sequence[0], Rule)
        assert r.sequence[0].type == "str"

        # Test short name 'seq'
        r = Rule(schema={"seq": [{"type": "str"}]})
        assert r.type == "seq"
        assert isinstance(r.sequence, list)
        assert isinstance(r.sequence[0], Rule)
        assert r.sequence[0].type == "str"

        # Test error is raised when sequence key is missing
        with pytest.raises(SchemaConflict) as ex:
            Rule(schema={"type": "seq"})
        assert str(ex.value) == "<SchemaConflict: error code 5: Type is sequence but no sequence alias found on same level: Path: '/'>"

        # sequence and pattern can't be used at same time
        with pytest.raises(SchemaConflict) as ex:
            Rule(schema={"type": "seq", "sequence": [{"type": "str"}], "pattern": "..."})
        assert str(ex.value) == "<SchemaConflict: error code 5: Sequence and pattern can't be on the same level in the schema: Path: '/'>"

    def test_build_sequence_multiple_values(self):
        """
        Test with multiple values.
        """
        # Test basic sequence rule
        r = Rule(schema={'type': 'seq', 'sequence': [{'type': 'str'}, {'type': 'int'}]})
        assert r.type == "seq"
        assert r.matching == "any"
        assert len(r.sequence) == 2
        assert isinstance(r.sequence, list)
        assert all(isinstance(r.sequence[i], Rule) for i in range(len(r.sequence)))
        assert r.sequence[0].type == "str"
        assert r.sequence[1].type == "int"

        # Test sequence without explicit type
        r = Rule(schema={'sequence': [{'type': 'str'}, {'type': 'int'}]})
        assert r.type == "seq"
        assert r.matching == "any"
        assert len(r.sequence) == 2
        assert isinstance(r.sequence, list)
        assert all(isinstance(r.sequence[i], Rule) for i in range(len(r.sequence)))
        assert r.sequence[0].type == "str"
        assert r.sequence[1].type == "int"

        # Test adding matchin rules

    def test_mapping(self):
        # This tests mapping with a nested type and pattern
        r = Rule(schema={"type": "map", "mapping": {"name": {"type": "str", "pattern": ".+@.+"}}})
        assert r.type == "map", "rule type is not map"
        assert isinstance(r.mapping, dict), "mapping is not dict"
        assert r.mapping["name"].type == "str", "nested mapping is not of string type"
        assert r.mapping["name"].pattern is not None, "nested mapping has no pattern var set"
        assert r.mapping["name"].pattern == ".+@.+", "pattern is not set to correct value"

        # when type is specefied, 'mapping' key must be present
        with pytest.raises(SchemaConflict) as ex:
            Rule(schema={"type": "map"})
        assert str(ex.value) == "<SchemaConflict: error code 5: Type is mapping but no mapping alias found on same level: Path: '/'>"

        # 'map' and 'enum' can't be used at same time
        # TODO: This do not work because it currently raises RuleError: <RuleError: error code 4: enum.notscalar>
        # with pytest.raises(SchemaConflict):
        #     r = Rule(schema={"type": "map", "enum": [1, 2, 3]})

        # Test that 'map' and 'mapping' can't be at the same level
        with pytest.raises(RuleError) as r:
            Rule(schema={"map": {"stream": {"type": "any"}}, "mapping": {"seams": {"type": "any"}}})
        assert str(r.value) == "<RuleError: error code 4: Keywords 'map' and 'mapping' can't be used on the same level: Path: '/'>"
        assert r.value.error_key == 'mapping.duplicate_keywords'

        # This will test that a invalid regex will throw error when parsing rules
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "map", "matching-rule": "any", "mapping": {"regex;(+": {"type": "seq", "sequence": [{"type": "str"}]}}})
        assert str(r.value) == "<RuleError: error code 4: Unable to compile regex '(+': Path: '/'>"
        assert r.value.error_key == 'mapping.regex.compile_error'

        # this tests map/dict but with no elements
        with pytest.raises(RuleError) as r:
            Rule(schema={"type": "map", "mapping": {}})
        assert str(r.value) == "<RuleError: error code 4: Mapping do not contain any elements: Path: '/'>"
        assert r.value.error_key == 'mapping.no_elements'

    def test_default_value(self):
        pass

    def test_check_conflicts(self):
        # TODO: This do not work and enum schema conflict is not raised... RuleError: <RuleError: error code 4: enum.notscalar>
        # with pytest.raises(SchemaConflict) as ex:
        #     r = Rule(schema={"type": "seq", "sequence": [{"type": "str"}], "enum": [1, 2, 3]})
        # assert ex.value.msg.startswith("seq.conflict :: enum"), "Wrong exception was raised"

        # Test sequence and mapping can't be used at same level
        with pytest.raises(SchemaConflict) as ex:
            Rule(schema={"type": "seq", "sequence": [{"type": "str"}], "mapping": {"name": {"type": "str", "pattern": ".+@.+"}}})
        assert str(ex.value) == "<SchemaConflict: error code 5: Sequence and mapping can't be on the same level in the schema: Path: '/'>"
        assert ex.value.error_key == 'seq.conflict.mapping'

        # Mapping and sequence can't used at same time
        with pytest.raises(SchemaConflict) as ex:
            Rule(schema={"type": "map", "mapping": {"foo": {"type": "str"}}, "sequence": [{"type": "str"}]})
        assert str(ex.value) == "<SchemaConflict: error code 5: Mapping and sequence can't be on the same level in the schema: Path: '/'>"
        assert ex.value.error_key == 'map.conflict.sequence'

        # scalar type and sequence can't be used at same time
        with pytest.raises(SchemaConflict) as ex:
            Rule(schema={"type": "int", "sequence": [{"type": "str"}]})
        assert str(ex.value) == "<SchemaConflict: error code 5: Scalar and sequence can't be on the same level in the schema: Path: '/'>"
        assert ex.value.error_key == 'scalar.conflict.sequence'

        # scalar type and mapping can't be used at same time
        with pytest.raises(SchemaConflict) as ex:
            Rule(schema={"type": "int", "mapping": {"foo": {"type": "str"}}})
        assert str(ex.value) == "<SchemaConflict: error code 5: Scalar and mapping can't be on the same level in the schema: Path: '/'>"
        assert ex.value.error_key == 'scalar.conflict.mapping'

        # scalar type and enum can't be used at same time
        with pytest.raises(SchemaConflict) as ex:
            Rule(schema={"type": "int", "enum": [1, 2, 3], "range": {"max": 10, "min": 1}})
        assert str(ex.value) == "<SchemaConflict: error code 5: Enum and range can't be on the same level in the schema: Path: '/'>"
        assert ex.value.error_key == 'enum.conflict.range'



# -*- coding: utf-8 -*-

# python std lib
import logging
import logging.config

log = logging.getLogger()


# Set the root logger to be silent so all code that uses the python logger
# will not print anything unless we want it to, then it should be specified
# in each test and reseted after that test
def _set_log_lv(level=1337, loggers=None):
    """ If no level is set then level will be so high all logging is silenced
    """
    if loggers is None:
        # If no additional loggers is specified then only apply to root logger
        log.setLevel(level)
        for handler in log.handlers:
            handler.level = level
    else:
        # If we have other logging instances specified apply to root logger and them
        if log not in loggers:
            loggers.append(log)

        for log_instance in loggers:
            log_instance.setLevel(level)
            for handler in log_instance.handlers:
                handler.level = level


# Initially silence all logging
_set_log_lv()



# -*- coding: utf-8 -*-

# pykwalify imports
from pykwalify import errors


class TestCLI(object):

    def test_base_exception(self):
        # retcode=2 should be schemaerror
        e = errors.PyKwalifyException(msg="foobar", retcode=2)
        assert e.__repr__() == "PyKwalifyException(msg='foobar')"
        assert e.retname == "schemaerror"

    def test_create_sub_class_exceptions(self):
        u_e = errors.UnknownError()
        assert u_e.retcode == 1

        s_e = errors.SchemaError()
        assert s_e.retcode == 2

        c_e = errors.CoreError()
        assert c_e.retcode == 3

        r_e = errors.RuleError()
        assert r_e.retcode == 4

        sc_e = errors.SchemaConflict()
        assert sc_e.retcode == 5



# -*- coding: utf-8 -*-
import pytest
from datetime import datetime

from pykwalify.compat import unicode
from pykwalify.core import Core
# from pykwalify.errors import NotSequenceError, CoreError
from pykwalify.errors import CoreError


class Rule(object):
    def __init__(self, sequence=None, mapping=None, rule_type=None):
        self.sequence = sequence or []
        self.mapping = mapping or {}
        self.type = rule_type or ''


def _remap_errors(c):
    return [unicode(error) for error in c.errors]


# TODO: Refactor this becuase it no longer raises NotSequenceError but it now adds an error to the
#       error stack and it should look for that one instead.
# def test_validate_sequence():
#     # If the type is set to sequence but value is int, it should raise NotSequenceError
#     with pytest.raises(NotSequenceError):
#         c = Core(source_data={}, schema_data={})
#         c._validate_sequence(123, Rule(sequence=['']), '', [])


def ec():
    # Return a empty core object
    return Core(source_data={}, schema_data={})


def test_validate_range():
    data_matrix = [
        (10, 5, 10, 5, 7, []),
        (None, None, None, None, 7, []),

        (10, 5, None, None, 13, ["Type 'prefix' has size of '13', greater than max limit '10'. Path: '/'"]),
        (10, 5, None, None, 3, ["Type 'prefix' has size of '3', less than min limit '5'. Path: '/'"]),
        (10, 5, None, None, 13.5, ["Type 'prefix' has size of '13.5', greater than max limit '10'. Path: '/'"]),
        (10, 5, None, None, 3.5, ["Type 'prefix' has size of '3.5', less than min limit '5'. Path: '/'"]),
        (10, 5, None, None, 10, []),
        (10, 5, None, None, 5, []),
        (10, 5, None, None, 10.0, []),
        (10, 5, None, None, 5.0, []),

        (None, None, 10, 5, 13, ["Type 'prefix' has size of '13', greater than or equals to max limit(exclusive) '10'. Path: '/'"]),
        (None, None, 10, 5, 3, ["Type 'prefix' has size of '3', less than or equals to min limit(exclusive) '5'. Path: '/'"]),
        (None, None, 10, 5, 13.5, ["Type 'prefix' has size of '13.5', greater than or equals to max limit(exclusive) '10'. Path: '/'"]),
        (None, None, 10, 5, 3.5, ["Type 'prefix' has size of '3.5', less than or equals to min limit(exclusive) '5'. Path: '/'"]),
        (None, None, 10, 5, 10, ["Type 'prefix' has size of '10', greater than or equals to max limit(exclusive) '10'. Path: '/'"]),
        (None, None, 10, 5, 5, ["Type 'prefix' has size of '5', less than or equals to min limit(exclusive) '5'. Path: '/'"]),
        (None, None, 10, 5, 8, []),
        (None, None, 10, 5, 7, []),
        (None, None, 10, 5, 8.5, []),
        (None, None, 10, 5, 7.5, []),
    ]

    for max_, min_, max_ex, min_ex, value, errors in data_matrix:
        print(u"Testing data: {0} {1} {2} {3} {4}".format(max_, min_, max_ex, min_ex, value))
        c = ec()
        c._validate_range(max_, min_, max_ex, min_ex, value, '/', 'prefix')
        assert _remap_errors(c) == errors

    # Test value type validation inside the method
    with pytest.raises(CoreError):
        c = ec()
        c._validate_range(5, 1, None, None, [1, 2, 3], '/', 'prefix')

    with pytest.raises(CoreError):
        c = ec()
        c._validate_range(5, 1, None, None, {'a': 1, 'b': 2, 'c': 3}, '/', 'prefix')


def test_validate_timestamp():
    data_matrix = [
        ("", ["Timestamp value is empty. Path: ''"]),
        ("1234567", []),
        ("2016-01-01", []),
        ("2016-01-01 15:01", []),
        (123, []),
        (1.5, []),
        (0, ["Integer value of timestamp can't be below 0"]),
        (-1, ["Integer value of timestamp can't be below 0"]),
        (3147483647, ["Integer value of timestamp can't be above 2147483647"]),
        ([], ["Not a valid timestamp"]),
        (datetime.now(), []),
        (datetime.today(), []),
    ]

    for data in data_matrix:
        c = ec()
        c._validate_scalar_timestamp(data[0], '')
        assert _remap_errors(c) == data[1]


def test_validate_date():
    formats = ["%Y-%m-%d"]

    data_matrix = [
        (datetime.now(), [[], []]),
        (datetime.today(), [[], []]),
        ("1234567", [["Not a valid date: 1234567 format: %Y-%m-%d. Path: ''"], []]),
        ("2016-01-01", [[], []]),
        ("2016-01-01 15:01", [["Not a valid date: 2016-01-01 15:01 format: %Y-%m-%d. Path: ''"], []]),
        (-1, [["Not a valid date: -1 date must be a string or a datetime.date not a 'int'"], []]),
        (0, [["Not a valid date: 0 date must be a string or a datetime.date not a 'int'"], []]),
        (1.5, [["Not a valid date: 1.5 date must be a string or a datetime.date not a 'float'"], []]),
        (3147483647, [["Not a valid date: 3147483647 date must be a string or a datetime.date not a 'int'"], []]),
        ([], [["Not a valid date: [] date must be a string or a datetime.date not a 'list'"], []]),
        ({}, [["Not a valid date: {} date must be a string or a datetime.date not a 'dict'"], []]),
    ]

    for data in data_matrix:
        for i, format in enumerate(formats):
            print("Validating: {0} Format: {1}".format(data[0], format))
            c = ec()
            c._validate_scalar_date(data[0], [format], '')
            assert _remap_errors(c) == data[1][i]


def test_validate_scalar_type():
    # Test that when providing a scalar type that do not exists, it should raise an error
    with pytest.raises(CoreError):
        c = ec()
        c._validate_scalar_type(True, True, '')

    data_matrix = []

    # Tests for str
    data_matrix += [
        ("", "str", []),
        ("123", "str", []),
        ("yes", "str", []),
        ("no", "str", []),
        (b"foobar", "str", []),
        (u"Néron", "str", []),
        (123, "str", ["Value '123' is not of type 'str'. Path: ''"]),
        (None, "str", ["Value 'None' is not of type 'str'. Path: ''"]),
        (3.14, "str", ["Value '3.14' is not of type 'str'. Path: ''"]),
        (True, "str", ["Value 'True' is not of type 'str'. Path: ''"]),
        ({'a': 'b'}, "str", ["Value '{'a': 'b'}' is not of type 'str'. Path: ''"]),
        (['a', 'b'], "str", ["Value '['a', 'b']' is not of type 'str'. Path: ''"]),
    ]

    # Tests for int
    data_matrix += [
        (123, "int", []),
        (3.14, "int", ["Value '3.14' is not of type 'int'. Path: ''"]),
        ("", "int", ["Value '' is not of type 'int'. Path: ''"]),
        ("123", "int", ["Value '123' is not of type 'int'. Path: ''"]),
        # (b"foobar", "int", ["Value b'foobar' is not of type 'int'. Path: ''"]),
        (u"Néron", "int", [u"Value 'Néron' is not of type 'int'. Path: ''"]),
        (None, "int", ["Value 'None' is not of type 'int'. Path: ''"]),
        (True, "int", ["Value 'True' is not of type 'int'. Path: ''"]),
        ({'a': 'b'}, "int", ["Value '{'a': 'b'}' is not of type 'int'. Path: ''"]),
        (['a', 'b'], "int", ["Value '['a', 'b']' is not of type 'int'. Path: ''"]),
    ]

    # Tests for float type
    data_matrix += [
        ("1e-06", 'float', []),
        ("1z-06", 'float', ["Value '1z-06' is not of type 'float'. Path: ''"]),
        (1.5, 'float', []),
        ("abc", 'float', ["Value 'abc' is not of type 'float'. Path: ''"]),
        # (b"abc", 'float', ["Value 'abc' is not of type 'float'. Path: ''"]),
        (u"abc", 'float', ["Value 'abc' is not of type 'float'. Path: ''"]),
        (True, 'float', ["Value 'True' is not of type 'float'. Path: ''"]),
    ]

    # Tests for bool
    data_matrix += [
        (True, "bool", []),
        (False, "bool", []),
        (1, "bool", ["Value '1' is not of type 'bool'. Path: ''"]),
        (3.14, "bool", ["Value '3.14' is not of type 'bool'. Path: ''"]),
        ("", "bool", ["Value '' is not of type 'bool'. Path: ''"]),
        ("yes", "bool", ["Value 'yes' is not of type 'bool'. Path: ''"]),
        ("no", "bool", ["Value 'no' is not of type 'bool'. Path: ''"]),
        # (b"foobar", "bool", [b"Value 'foobar' is not of type 'bool'. Path: ''"]),
        (u"Néron", "bool", [u"Value 'Néron' is not of type 'bool'. Path: ''"]),
        ([], "bool", ["Value '[]' is not of type 'bool'. Path: ''"]),
        ({}, "bool", ["Value '{}' is not of type 'bool'. Path: ''"]),
    ]

    # Tests for number
    data_matrix += [
        (1, "number", []),
        (3.14, "number", []),
        (True, "number", ["Value 'True' is not of type 'number'. Path: ''"]),
        (False, "number", ["Value 'False' is not of type 'number'. Path: ''"]),
        ("", "number", ["Value '' is not of type 'number'. Path: ''"]),
        ("yes", "number", ["Value 'yes' is not of type 'number'. Path: ''"]),
        ("no", "number", ["Value 'no' is not of type 'number'. Path: ''"]),
        # (b"foobar", "number", [b"Value 'foobar' is not of type 'number'. Path: ''"]),
        (u"Néron", "number", [u"Value 'Néron' is not of type 'number'. Path: ''"]),
        ([], "number", ["Value '[]' is not of type 'number'. Path: ''"]),
        ({}, "number", ["Value '{}' is not of type 'number'. Path: ''"]),
    ]

    # Tests for text
    data_matrix += [
        (1, "text", []),
        (3.14, "text", []),
        ("", "text", []),
        ("yes", "text", []),
        ("no", "text", []),
        # (b"foobar", "text", []),
        (u"Néron", "text", []),
        (True, "text", ["Value 'True' is not of type 'text'. Path: ''"]),
        (False, "text", ["Value 'False' is not of type 'text'. Path: ''"]),
        ([], "text", ["Value '[]' is not of type 'text'. Path: ''"]),
        ({}, "text", ["Value '{}' is not of type 'text'. Path: ''"]),
        (datetime(2015, 10, 24, 10, 22, 18), "text", ["Value '2015-10-24 10:22:18' is not of type 'text'. Path: ''"]),
    ]

    # Tests for any
    data_matrix += [
        (1, "any", []),
        (3.14, "any", []),
        (True, "any", []),
        (False, "any", []),
        ("", "any", []),
        ("yes", "any", []),
        ("no", "any", []),
        # (b"foobar", "any", []),
        (u"Néron", "any", []),
        ([], "any", []),
        ({}, "any", []),
        (datetime(2015, 10, 24, 10, 22, 18), "any", []),
    ]

    # Tests for enum
    data_matrix += [
        ("", "enum", []),
        ("123", "enum", []),
        ("yes", "enum", []),
        ("no", "enum", []),
        # (b"foobar", "enum", []),
        (u"Néron", "enum", []),
        (123, "enum", ["Value '123' is not of type 'enum'. Path: ''"]),
        (None, "enum", ["Value 'None' is not of type 'enum'. Path: ''"]),
        (3.14, "enum", ["Value '3.14' is not of type 'enum'. Path: ''"]),
        (True, "enum", ["Value 'True' is not of type 'enum'. Path: ''"]),
        ({'a': 'b'}, "enum", ["Value '{'a': 'b'}' is not of type 'enum'. Path: ''"]),
        (['a', 'b'], "enum", ["Value '['a', 'b']' is not of type 'enum'. Path: ''"]),
    ]

    # Tests for none
    data_matrix += [
        ("", "none", ["Value '' is not of type 'none'. Path: ''"]),
        ("123", "none", ["Value '123' is not of type 'none'. Path: ''"]),
        ("yes", "none", ["Value 'yes' is not of type 'none'. Path: ''"]),
        ("no", "none", ["Value 'no' is not of type 'none'. Path: ''"]),
        ("None", "none", ["Value 'None' is not of type 'none'. Path: ''"]),
        # (b"foobar", "none", [b"Value 'foobar' is not of type 'none'. Path: ''"]),
        (u"Néron", "none", [u"Value 'Néron' is not of type 'none'. Path: ''"]),
        (123, "none", ["Value '123' is not of type 'none'. Path: ''"]),
        (None, "none", []),
        (3.14, "none", ["Value '3.14' is not of type 'none'. Path: ''"]),
        (True, "none", ["Value 'True' is not of type 'none'. Path: ''"]),
        ({'a': 'b'}, "none", ["Value '{'a': 'b'}' is not of type 'none'. Path: ''"]),
        (['a', 'b'], "none", ["Value '['a', 'b']' is not of type 'none'. Path: ''"]),

    ]

    # Tests for timestamp
    data_matrix += [
        ("", 'timestamp', []),
        ("1234567", 'timestamp', []),
        ("2016-01-01", 'timestamp', []),
        ("2016-01-01 15:01", 'timestamp', []),
        # (b"foobar", "timestamp", []),
        (u"Néron", "timestamp", []),
        (123, 'timestamp', []),
        (1.5, 'timestamp', []),
        (0, 'timestamp', []),
        (-1, 'timestamp', []),
        (3147483647, 'timestamp', []),
        ([], 'timestamp', ["Value '[]' is not of type 'timestamp'. Path: ''"]),
        (datetime.now(), 'timestamp', []),
        (datetime.today(), 'timestamp', []),
    ]

    data_matrix += [
        (datetime(2015, 10, 24, 10, 22, 18), 'scalar', []),
        ("", "scalar", []),
        ("2016-01-01", 'scalar', []),
        ("2016-01-01 15:01", 'scalar', []),
        ("123", 'scalar', []),
        ("yes", 'scalar', []),
        (u"Néron", 'scalar', []),
        (None, 'scalar', ["Value 'None' is not of type 'scalar'. Path: ''"]),
        (123, 'scalar', []),
        (3.14, 'scalar', []),
        (True, 'scalar', []),
        ({'a': 'b'}, 'scalar', ["Value '{'a': 'b'}' is not of type 'scalar'. Path: ''"]),
        (['a', 'b'], 'scalar', ["Value '['a', 'b']' is not of type 'scalar'. Path: ''"]),
    ]

    for data in data_matrix:
        print(u"Testing data: '{0!s}', '{1!s}', '{2!s}'".format(*data))
        c = ec()
        c._validate_scalar_type(data[0], data[1], '')
        assert _remap_errors(c) == data[2]



# -*- coding: utf-8 -*-

""" Unit test for pyKwalify - Rule """

# python std lib
import unittest

# pykwalify imports
from pykwalify import types


class TestTypes(unittest.TestCase):

    def test_types(self):
        """
        Test that all type helper methods works correctly
        """
        assert types.type_class("str") == str

        assert types.is_builtin_type("str")

        assert types.is_collection_type("map")
        assert types.is_collection_type("seq")
        assert not types.is_collection_type("str")

        assert types.is_scalar_type("str")
        assert not types.is_scalar_type("seq")
        assert not types.is_scalar_type("map")

        assert types.is_collection([])
        assert types.is_collection({})
        assert not types.is_collection("foo")

        assert types.is_scalar("")
        assert types.is_scalar(True)
        assert not types.is_scalar([])

        assert types.is_correct_type("", str)
        assert types.is_correct_type({}, dict)

        assert types.is_string("foo")
        assert not types.is_string([])

        assert types.is_int(1)
        assert not types.is_int("foo")

        assert types.is_bool(True)
        assert not types.is_bool(1)
        assert not types.is_bool("true")

        assert types.is_float(1.0)
        assert not types.is_float("foo")

        assert types.is_number(1)
        assert types.is_number(1.0)
        assert not types.is_number("foo")

        assert types.is_text("foo")
        assert types.is_text(1)
        assert types.is_text(1.0)
        assert not types.is_text([])
        assert not types.is_text(True)

        assert types.is_any("foo")
        assert types.is_any(True)
        assert types.is_any(1)
        assert types.is_any(1.0)
        assert types.is_any({})
        assert types.is_any([])

        assert types.is_enum("foo")
        assert not types.is_enum(1)

        assert types.is_none(None)
        assert not types.is_none("foo")



# -*- coding: utf-8 -*-

""" Unit test for pyKwalify - Core """

# python std lib
import os

# pykwalify imports
import pykwalify
from pykwalify.core import Core
from pykwalify.errors import SchemaError, CoreError

# 3rd party imports
import pytest
from pykwalify.compat import yaml
from testfixtures import compare


class TestCore(object):

    def setUp(self):
        pykwalify.partial_schemas = {}

    def f(self, *args):
        return os.path.join(os.path.dirname(os.path.realpath(__file__)), "files", *args)

    def test_create_empty_core_object(self, tmpdir):
        """
        If createing a core object without any source or schema file an exception should be raised.
        """
        with pytest.raises(CoreError) as ex:
            Core()
        assert "No source file/data was loaded" in str(ex.value)

        # To trigger schema exception we must pass in a source file
        source_f = tmpdir.join("bar.json")
        source_f.write("3.14159")

        with pytest.raises(CoreError) as ex:
            Core(source_file=str(source_f))
        assert "No schema file/data was loaded" in str(ex.value)

    def test_load_non_existing_file(self):
        file_to_load = "/tmp/foo/bar/barfoo"
        assert not os.path.exists(file_to_load), "Following file cannot exists on your system while running these tests : {0}".format(file_to_load)
        with pytest.raises(CoreError) as ex:
            Core(source_file=file_to_load)
        assert "Provided source_file do not exists on disk" in str(ex.value)

    def test_load_non_existsing_schema_file(self):
        """
        Exception should be raised if the specefied schema file do not exists on disk.
        """
        file_to_load = "/tmp/foo/bar/barfoo"
        assert not os.path.exists(file_to_load), "Following file cannot exists on your system while running these tests : {0}".format(file_to_load)
        with pytest.raises(CoreError) as ex:
            Core(schema_files=[file_to_load])
        assert "Provided source_file do not exists on disk" in str(ex.value)

    def test_load_wrong_schema_files_type(self):
        """
        It should only be possible to send in a list type as 'schema_files' object
        """
        with pytest.raises(CoreError) as ex:
            Core(source_file=None, schema_files={})
        assert "schema_files must be of list type" in str(ex.value)

    def test_load_json_file(self, tmpdir):
        """
        Load source & schema files that has json file ending.
        """
        source_f = tmpdir.join("bar.json")
        source_f.write("3.14159")

        schema_f = tmpdir.join("foo.json")
        schema_f.write('{"type": "float"}')

        Core(source_file=str(source_f), schema_files=[str(schema_f)])

        # TODO: Try to load a non existing json file

    def test_load_yaml_files(self, tmpdir):
        """
        Load source & schema files that has yaml file ending.
        """
        source_f = tmpdir.join("foo.yaml")
        source_f.write("3.14159")

        schema_f = tmpdir.join("bar.yaml")
        schema_f.write("type: float")

        Core(source_file=str(source_f), schema_files=[str(schema_f)])

    def test_load_unsupported_format(self, tmpdir):
        """
        Try to load some fileending that is not supported.
        Currently XML is not supported.
        """
        source_f = tmpdir.join("foo.xml")
        source_f.write("<foo>bar</foo>")

        schema_f = tmpdir.join("bar.xml")
        schema_f.write("<foo>bar</foo>")

        with pytest.raises(CoreError) as ex:
            Core(source_file=str(source_f))
        assert "Unable to load source_file. Unknown file format of specified file path" in str(ex.value)

        with pytest.raises(CoreError) as ex:
            Core(schema_files=[str(schema_f)])
        assert "Unknown file format. Supported file endings is" in str(ex.value)

    def test_load_empty_json_file(self, tmpdir):
        """
        Loading an empty json files should raise an exception
        """
        # Load empty source file
        source_f = tmpdir.join("foo.json")
        source_f.write("")

        schema_f = tmpdir.join("bar.json")
        schema_f.write("")

        with pytest.raises(ValueError) as ex:
            Core(source_file=str(source_f), schema_files=[str(schema_f)])
        # Python 2.7 and Python 3.5 JSON parsers return different exception
        # strings for the same data file, so check for both errors strings.
        assert ("No JSON object could be decoded" in str(ex.value) or
                "Expecting value:" in str(ex.value))

        # Load empty schema files
        source_f = tmpdir.join("foo.json")
        source_f.write("3.14159")

        schema_f = tmpdir.join("bar.json")
        schema_f.write("")

        with pytest.raises(ValueError) as ex:
            Core(source_file=str(source_f), schema_files=[str(schema_f)])
        assert ("No JSON object could be decoded" in str(ex.value) or
                "Expecting value:" in str(ex.value))

    def test_load_empty_yaml_file(self, tmpdir):
        """
        Loading empty yaml files should raise an exception
        """
        # Load empty source file
        source_f = tmpdir.join("foo.yaml")
        source_f.write("")

        schema_f = tmpdir.join("bar.yaml")
        schema_f.write("")

        # TODO: This is abit buggy because wrong exception is raised...
        with pytest.raises(CoreError) as ex:
            Core(source_file=str(source_f), schema_files=[str(schema_f)])
        # assert "Unable to load any data from source yaml file" in str(ex.value)

        # Load empty schema files
        source_f = tmpdir.join("foo.yaml")
        source_f.write("3.14159")

        schema_f = tmpdir.join("bar.yaml")
        schema_f.write("")

        with pytest.raises(CoreError) as ex:
            Core(source_file=str(source_f), schema_files=[str(schema_f)])
        assert "No data loaded from file" in str(ex.value)

    def test_validation_error_but_not_raise_exception(self):
        """
        Test that if 'raise_exception=False' when validating that no exception is raised.

        Currently file 2a.yaml & 2b.yaml is designed to cause exception.
        """
        c = Core(source_file=self.f("cli", "2a.yaml"), schema_files=[self.f("cli", "2b.yaml")])
        c.validate(raise_exception=False)

        assert c.validation_errors == [
            "Value '1' is not of type 'str'. Path: '/0'", "Value '2' is not of type 'str'. Path: '/1'", "Value '3' is not of type 'str'. Path: '/2'"
        ]

        # TODO: Fix this issue...
        # assert ('pykwalify.core', 'ERROR', 'Errors found but will not raise exception...') in l.actual()

    def test_core_data_mode(self):
        Core(source_data=3.14159, schema_data={"type": "number"}).validate()
        Core(source_data="1e-06", schema_data={"type": "float"}).validate()
        Core(source_data=3.14159, schema_data={"type": "float"}).validate()
        Core(source_data=3, schema_data={"type": "float"}).validate()
        Core(source_data=3, schema_data={"type": "int"}).validate()
        Core(source_data=True, schema_data={"type": "bool"}).validate()
        Core(source_data="foobar", schema_data={"type": "str"}).validate()
        Core(source_data="foobar", schema_data={"type": "text"}).validate()
        Core(source_data="foobar", schema_data={"type": "any"}).validate()

        # Test that 'any' allows types that is not even implemented
        def foo():
            pass
        Core(source_data=foo, schema_data={"type": "any"}).validate()
        Core(source_data=lambda x: x, schema_data={"type": "any"}).validate()

        with pytest.raises(SchemaError):
            Core(source_data="1z-06", schema_data={"type": "float"}).validate()

        with pytest.raises(SchemaError):
            Core(source_data="abc", schema_data={"type": "number"}).validate()

        with pytest.raises(SchemaError):
            Core(source_data=3.14159, schema_data={"type": "int"}).validate()

        with pytest.raises(SchemaError):
            Core(source_data=1337, schema_data={"type": "bool"}).validate()

        with pytest.raises(SchemaError):
            Core(source_data=1, schema_data={"type": "str"}).validate()

        with pytest.raises(SchemaError):
            Core(source_data=True, schema_data={"type": "text"}).validate()

    def test_multi_file_support(self):
        """
        This should test that multiple files is supported correctly
        """
        pass_tests = [
            # Test that include directive can be used at top level of the schema
            (
                [
                    self.f("partial_schemas", "1s-schema.yaml"),
                    self.f("partial_schemas", "1s-partials.yaml"),
                ],
                self.f("partial_schemas", "1s-data.yaml"),
                {
                    'sequence': [{'include': 'fooone'}],
                    'type': 'seq',
                }
            ),
            # # This test that include directive works inside sequence
            # ([self.f("33a.yaml"), self.f("33b.yaml")], self.f("33c.yaml"), {'sequence': [{'include': 'fooone'}], 'type': 'seq'}),
            # This test recursive schemas
            (
                [
                    self.f("partial_schemas", "2s-schema.yaml"),
                    self.f("partial_schemas", "2s-partials.yaml"),
                ],
                self.f("partial_schemas", "2s-data.yaml"),
                {
                    'sequence': [{'include': 'fooone'}],
                    'type': 'seq',
                }
            ),
            # This tests that you can include a partial schema alongside other rules in a map
            (
                [
                    self.f("partial_schemas", "7s-schema.yaml"),
                ],
                self.f("partial_schemas", "7s-data.yaml"),
                {
                    'type': 'map',
                    'mapping': {
                        'foo': {
                            'type': 'str',
                            'required': True
                        },
                        'bar': {
                            'include': 'bar'
                        }
                    }
                }
            )
        ]

        failing_tests = [
            # Test include inside partial schema
            (
                [
                    self.f("partial_schemas", "1f-schema.yaml"),
                    self.f("partial_schemas", "1f-partials.yaml")
                ],
                self.f("partial_schemas", "1f-data.yaml"),
                SchemaError,
                ["Cannot find partial schema with name 'fooonez'. Existing partial schemas: 'bar, fooone, foothree, footwo'. Path: '/0'"]
            ),
            (
                [
                    self.f('partial_schemas', '2f-schema.yaml')
                ],
                self.f('partial_schemas', '2f-data.yaml'),
                SchemaError,
                ["Value 'True' is not of type 'str'. Path: '/0'"]
            ),
            (
                [
                    self.f('partial_schemas', '3f-schema.yaml')
                ],
                self.f('partial_schemas', '3f-data.yaml'),
                SchemaError,
                ["Value 'True' is not of type 'str'. Path: ''"]
            ),
            (
                [
                    self.f('partial_schemas', '4f-schema.yaml')
                ],
                self.f('partial_schemas', '4f-data.yaml'),
                SchemaError,
                ["Value 'True' is not of type 'str'. Path: '/0/foo/0/bar'"]
            ),
            (
                [
                    self.f('partial_schemas', '5f-schema.yaml')
                ],
                self.f('partial_schemas', '5f-data.yaml'),
                SchemaError,
                ["Value 'True' is not of type 'str'. Path: '/0/0/0/0'"]
            ),
            (
                [
                    self.f('partial_schemas', '6f-schema.yaml')
                ],
                self.f('partial_schemas', '6f-data.yaml'),
                SchemaError,
                ["Value 'True' is not of type 'str'. Path: '/foo/bar/qwe/ewq'"]
            )
        ]

        for passing_test in pass_tests:
            try:
                c = Core(source_file=passing_test[1], schema_files=passing_test[0])
                c.validate()
                compare(c.validation_errors, [], prefix="No validation errors should exist...")
            except Exception as e:
                print("ERROR RUNNING FILE: {0} : {1}".format(passing_test[0], passing_test[1]))
                raise e

            # This serve as an extra schema validation that tests more complex structures then testrule.py do
            compare(c.root_rule.schema_str, passing_test[2], prefix="Parsed rules is not correct, something have changed...")

        for failing_test in failing_tests:
            with pytest.raises(failing_test[2], message="Test files: {0} : {1}".format(", ".join(failing_test[0]), failing_test[1])):
                c = Core(schema_files=failing_test[0], source_file=failing_test[1])
                c.validate()

            if not c.validation_errors:
                raise AssertionError("No validation_errors was raised...")

            compare(
                sorted(c.validation_errors),
                sorted(failing_test[3]),
                prefix="Wrong validation errors when parsing files : {0} : {1}".format(
                    failing_test[0],
                    failing_test[1],
                ),
            )

    def test_core_files(self):
        # These tests should pass with no exception raised
        pass_tests = [
            # All tests for keyword assert
            "test_assert.yaml",
            # All tests for keyword default
            "test_default.yaml",
            # All tests for keyword desc
            "test_desc.yaml",
            # All tests for keyword enum
            "test_enum.yaml",
            # All tests for keyword example
            "test_example.yaml",
            # All tests for keyword extensions
            "test_extensions.yaml",
            # All tests for keyword func
            "test_func.yaml",
            # All tests for keyword ident
            "test_ident.yaml",
            # All tests for keyword include
            "test_include.yaml",
            # All tests for keyword length
            "test_length.yaml",
            # All tests for keyword mapping
            "test_mapping.yaml",
            # All tests for keyword matching
            "test_matching.yaml",
            # All tests for keyword name
            "test_name.yaml",
            # All tests for keyword nullable
            "test_nullable.yaml",
            # All tests for keyword pattern
            "test_pattern.yaml",
            # All tests for keyword range
            "test_range.yaml",
            # All tests for keyword required
            "test_required.yaml",
            # All tests for keyword schema
            "test_schema.yaml",
            # All tests for keyword sequence
            "test_sequence.yaml",
            # All tests for keyword unique
            "test_unique.yaml",
            # All tests for keyword version
            "test_version.yaml",

            # All test cases for Multiple sequence checks
            "test_sequence_multi.yaml",
            # All test cases for merging
            "test_merge.yaml",
            # All test cases for yaml anchors
            "test_anchor.yaml",

            # All tests for TYPE: any
            "test_type_any.yaml",
            # All tests for TYPE: bool
            "test_type_bool.yaml",
            # All tests for TYPE: date
            "test_type_date.yaml",
            # All tests for TYPE: enum
            "test_type_enum.yaml",
            # All tests for TYPE: float
            "test_type_float.yaml",
            # All tests for TYPE: int
            "test_type_int.yaml",
            # All tests for TYPE: map
            "test_type_map.yaml",
            # All tests for TYPE: none
            "test_type_none.yaml",
            # All tests for TYPE: number
            "test_type_number.yaml",
            # All tests for TYPE: scalar
            "test_type_scalar.yaml",
            # All tests for TYPE: seq
            "test_type_seq.yaml",
            # All tests for TYPE: str
            "test_type_str.yaml",
            # All tests for TYPE: symbol
            "test_type_symbol.yaml",
            # All tests for TYPE: text
            "test_type_text.yaml",
            # All tests for TYPE: timestamp
            "test_type_timestamp.yaml",
        ]

        _fail_tests = [
            # All tests for keyword assert
            ("test_assert.yaml", SchemaError),
            # All tests for keyword default
            ("test_default.yaml", SchemaError),
            # All tests for keyword desc
            ("test_desc.yaml", SchemaError),
            # All tests for keyword enum
            ("test_enum.yaml", SchemaError),
            # All tests for keyword example
            ("test_example.yaml", SchemaError),
            # All tests for keyword extensions
            ("test_extensions.yaml", SchemaError),
            # All tests for keyword func
            ("test_func.yaml", SchemaError),
            # All tests for keyword ident
            ("test_ident.yaml", SchemaError),
            # All tests for keyword include
            ("test_include.yaml", SchemaError),
            # All tests for keyword length
            ("test_length.yaml", SchemaError),
            # All tests for keyword mapping
            ("test_mapping.yaml", SchemaError),
            # All tests for keyword matching
            ("test_matching.yaml", SchemaError),
            # All tests for keyword name
            ("test_name.yaml", SchemaError),
            # All tests for keyword nullable
            ("test_nullable.yaml", SchemaError),
            # All tests for keyword pattern
            ("test_pattern.yaml", SchemaError),
            # All tests for keyword range
            ("test_range.yaml", SchemaError),
            # All tests for keyword required
            ("test_required.yaml", SchemaError),
            # All tests for keyword schema
            ("test_schema.yaml", SchemaError),
            # All tests for keyword sequence
            ("test_sequence.yaml", SchemaError),
            # All tests for keyword unique
            ("test_unique.yaml", SchemaError),
            # All tests for keyword version
            ("test_version.yaml", SchemaError),

            # All test cases for Multiple sequence checks
            ("test_sequence_multi.yaml", SchemaError),
            # All test cases for merging
            ("test_merge.yaml", SchemaError),
            # All test cases for yaml anchors
            ("test_anchor.yaml", SchemaError),

            # All tests for TYPE: any
            ("test_type_any.yaml", SchemaError),
            # All tests for TYPE: bool
            ("test_type_bool.yaml", SchemaError),
            # All tests for TYPE: date
            ("test_type_date.yaml", SchemaError),
            # All tests for TYPE: float
            ("test_type_float.yaml", SchemaError),
            # All tests for TYPE: int
            ("test_type_int.yaml", SchemaError),
            # All tests for TYPE: map
            ("test_type_map.yaml", SchemaError),
            # All tests for TYPE: none
            ("test_type_none.yaml", SchemaError),
            # All tests for TYPE: number
            ("test_type_number.yaml", SchemaError),
            # All tests for TYPE: scalar
            ("test_type_scalar.yaml", SchemaError),
            # All tests for TYPE: seq
            ("test_type_seq.yaml", SchemaError),
            # All tests for TYPE: str
            ("test_type_str.yaml", SchemaError),
            # All tests for TYPE: symbol
            ("test_type_symbol.yaml", SchemaError),
            # All tests for TYPE: text
            ("test_type_text.yaml", SchemaError),
            # All tests for TYPE: timestamp
            ("test_type_timestamp.yaml", SchemaError),
        ]

        # Add override magic to make it easier to test a specific file
        if "S" in os.environ:
            pass_tests = [os.environ["S"]]
            _fail_tests = []
        elif "F" in os.environ:
            pass_tests = []
            _fail_tests = [(os.environ["F"], SchemaError)]

        for passing_test_file in pass_tests:
            f = self.f(os.path.join("success", passing_test_file))
            with open(f, "r") as stream:
                yaml_data = yaml.safe_load_all(stream)

                for document_index, document in enumerate(yaml_data):
                    data = document["data"]
                    schema = document["schema"]

                    try:
                        print("Running test files: {0}".format(f))
                        c = Core(source_data=data, schema_data=schema, strict_rule_validation=True, allow_assertions=True)
                        c.validate()
                        compare(c.validation_errors, [], prefix="No validation errors should exist...")
                    except Exception as e:
                        print("ERROR RUNNING FILES: {0} : {1}:{2}".format(f, document_index, document.get('name', 'UNKNOWN')))
                        raise e

            # This serve as an extra schema validation that tests more complex structures then testrule.py do
            compare(c.root_rule.schema_str, schema, prefix="Parsed rules is not correct, something have changed... files : {0} : {1}".format(f, document_index))

        for failing_test, exception_type in _fail_tests:
            f = self.f(os.path.join("fail", failing_test))
            with open(f, "r") as stream:
                yaml_data = yaml.safe_load_all(stream)

                for document_index, document in enumerate(yaml_data):
                    data = document["data"]
                    schema = document["schema"]
                    errors = document.get("errors", [])

                    try:
                        print("Running test files: {0}".format(f))
                        c = Core(source_data=data, schema_data=schema, strict_rule_validation=True, allow_assertions=True)
                        c.validate()
                    except exception_type as e:
                        pass
                    else:
                        print("ERROR RUNNING FILES: {0} : {1}:{2}".format(f, document_index, document.get('name', 'UNKNOWN')))
                        raise AssertionError("Exception {0} not raised as expected... FILES: {1} : {2} : {3}:{4}".format(
                            exception_type, exception_type, failing_test, document_index, document.get('name', 'UNKNOWN')))

                    compare(sorted(c.validation_errors), sorted(errors), prefix="Wrong validation errors when parsing files : {0} : {1} : {2}".format(
                        f, document_index, document.get('name', 'UNKNOWN')))



# -*- coding: utf-8 -*-

# python std lib
import os
import sys

# pykwalify package imports
from pykwalify import cli


class TestCLI(object):

    def test_cli(self, tmpdir):
        """
        Test that when passing in certain arguments from commandline they
        are handled correctly by docopt and correct args structure is returned.
        """
        input = tmpdir.join("cli/1a.yaml")
        schema_file = tmpdir.join("cli/1b.yaml")

        sys.argv = [
            'scripts/pykwalify',
            '-d', str(input),
            '-s', str(schema_file),
            '-v'
        ]

        expected = {
            '--data-file': str(input),
            '--schema-file': [str(schema_file)],
            '--quiet': False,
            '--verbose': 1,
        }

        cli_args = cli.parse_cli()

        for k, v in expected.items():
            assert k in cli_args
            assert cli_args[k] == expected[k]

    def f(self, *args):
        """
        Returns abs path to test files inside tests/files/
        """
        return os.path.join(os.path.dirname(os.path.realpath(__file__)), "files", *args)

    def test_run_cli(self):
        """
        This should test that running the cli still works as expected
        """
        input = self.f("cli/1a.yaml")
        schema_file = self.f("cli/1b.yaml")

        sys.argv = [
            'scripts/pykwalify',
            '-d', str(input),
            '-s', str(schema_file),
        ]

        cli_args = cli.parse_cli()
        c = cli.run(cli_args)
        assert c.validation_errors == []



# -*- coding: utf-8 -*-

""" Unit test for pyKwalify - Core """

# python std lib
import os

# pykwalify imports
import pykwalify
from pykwalify.compat import unicode
from pykwalify.core import Core
from pykwalify.errors import SchemaError

# 3rd party imports
from pykwalify.compat import yaml
from testfixtures import compare


class TestUnicode(object):

    def setUp(self):
        pykwalify.partial_schemas = {}

    def f(self, *args):
        if os.path.isabs(args[0]):
            return args[0]

        return os.path.join(os.path.dirname(os.path.realpath(__file__)), "files", "unicode", *args)

    def test_files_with_unicode_content_success(self, tmpdir):
        """
        These tests should pass with no exception raised
        """
        fail_data_2s_yaml = {
            'schema': {
                'type': 'map',
                'mapping': {
                    'msg': {
                        'type': 'int',
                    },
                }
            },
            'data': {
                'msg': 123,
            },
            'errors': []
        }

        source_f = tmpdir.join(u"2så.json")
        source_f.write(yaml.safe_dump(fail_data_2s_yaml, allow_unicode=True))

        _pass_tests = [
            # Test mapping with unicode key and value
            u"1s.yaml",
            # # Test unicode filename.
            # It is not possible to package a file with unicode characters
            # like åäö in the filename in some python versions.
            # Mock a file with åäö during testing to properly simulate this again.
            unicode(source_f),
            # Test sequence with unicode keys
            u"3s.yaml",
        ]

        for passing_test_files in _pass_tests:
            f = self.f(passing_test_files)

            with open(f, "r") as stream:
                yaml_data = yaml.safe_load(stream)
                data = yaml_data["data"]
                schema = yaml_data["schema"]

            try:
                print(u"Running test files: {0}".format(f))
                c = Core(source_data=data, schema_data=schema)
                c.validate()
                compare(c.validation_errors, [], prefix="No validation errors should exist...")
            except Exception as e:
                print(u"ERROR RUNNING FILES: {0}".format(f))
                raise e

            # This serve as an extra schema validation that tests more complex structures then testrule.py do
            compare(c.root_rule.schema_str, schema, prefix=u"Parsed rules is not correct, something have changed... files : {0}".format(f))

    def test_files_with_unicode_content_failing(self, tmpdir):
        """
        These tests should fail with the specified exception
        """
        # To trigger schema exception we must pass in a source file
        fail_data_2f_yaml = {
            'schema': {
                'type': 'map',
                'mapping': {
                    'msg': {
                        'type': 'int',
                    },
                }
            },
            'data': {
                'msg': 'Foobar',
            },
            'errors': ["Value 'Foobar' is not of type 'int'. Path: '/msg'"]
        }

        source_f = tmpdir.join(u"2få.json")
        source_f.write(yaml.safe_dump(fail_data_2f_yaml, allow_unicode=True))

        _fail_tests = [
            # Test mapping with unicode key and value but wrong type
            (u"1f.yaml", SchemaError),
            # Test unicode filename with validation errors.
            # It is not possible to package a file with unicode characters
            # like åäö in the filename in some python versions.
            # Mock a file with åäö during testing to properly simulate this again.
            (unicode(source_f), SchemaError),
            # Test unicode data inside seq but wrong type
            (u"3f.yaml", SchemaError),
        ]

        for failing_test, exception_type in _fail_tests:
            f = self.f(failing_test)

            with open(f, "r") as stream:
                yaml_data = yaml.safe_load(stream)
                data = yaml_data["data"]
                schema = yaml_data["schema"]
                errors = yaml_data["errors"]

            try:
                print(u"Running test files: {0}".format(f))
                c = Core(source_data=data, schema_data=schema)
                c.validate()
            except exception_type:
                pass  # OK
            else:
                raise AssertionError(u"Exception {0} not raised as expected... FILES: {1} : {2}".format(exception_type, exception_type))

            compare(sorted(c.validation_errors), sorted(errors), prefix=u"Wrong validation errors when parsing files : {0}".format(f))



import os
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open('README.md') as f:
    readme = f.read()
with open(os.path.join('docs', 'release-notes.rst')) as f:
    history = f.read()

setup(
    name="pykwalify",
    version="1.7.0",
    description='Python lib/cli for JSON/YAML schema validation',
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    author="Johan Andersson",
    author_email="Grokzen@gmail.com",
    maintainer='Johan Andersson',
    maintainer_email='Grokzen@gmail.com',
    license='MIT',
    packages=['pykwalify'],
    url='http://github.com/grokzen/pykwalify',
    extras_require={
        'ruamel': ["ruamel.yaml>=0.11.0,<0.16.0"],
    },
    entry_points={
        'console_scripts': [
            'pykwalify = pykwalify.cli:cli_entrypoint',
        ],
    },
    install_requires=[
        'docopt>=0.6.2',
        'PyYAML>=3.11',
        'python-dateutil>=2.4.2',
    ],
    classifiers=[
        # 'Development Status :: 1 - Planning',
        # 'Development Status :: 2 - Pre-Alpha',
        # 'Development Status :: 3 - Alpha',
        # 'Development Status :: 4 - Beta',
        'Development Status :: 5 - Production/Stable',
        # 'Development Status :: 6 - Mature',
        # 'Development Status :: 7 - Inactive',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Environment :: Console',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
